package com.me.brickbuster.entity;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.glutils.ShapeRenderer.ShapeType;
import com.badlogic.gdx.math.MathUtils;
import com.badlogic.gdx.math.Vector2;
import com.me.brickbuster.BrickBuster;
import com.me.brickbuster.Utils;
import net.dermetfan.utils.Pair;

import java.util.Iterator;

public class Ball extends Entity {

	public static final int RADIUS = 12;
	public static final Color BALL_COLOR = Color.CHARTREUSE;
	public static final float DEFAULT_SPEED = 350;
	public static final float BOOST_SPEED = 450;
	public static final int BLOCKS_FOR_BOOST = 39;

	public Vector2 direction;
	private float speed;
	private boolean isStuck = true;
	private boolean isDead = false;

	public Ball(BrickBuster brickBuster) {
		super(brickBuster,BrickBuster.WIDTH/2, Paddle.PADDLE_Y + Paddle.PADDLE_HEIGHT + RADIUS);
		this.speed = getBrickBuster().getBricks().size() > BLOCKS_FOR_BOOST? DEFAULT_SPEED : DEFAULT_SPEED;
	}

	@Override
	public void render() {
		getShapeRenderer().begin(ShapeType.Filled);
		getShapeRenderer().setColor(BALL_COLOR);
		getShapeRenderer().circle(getPos().x, getPos().y, RADIUS);
		getShapeRenderer().end();
	}

	@Override
	public void update(float dt) {
		if (isStuck || isDead) {
			if (!isDead && Gdx.input.justTouched()) {
				launch();
			} else {
				return;
			}
		}

		Vector2 new_pos = getPos().cpy().add(direction.cpy().scl(speed * dt));

		boolean brickCollision = false;
		Iterator<Brick> brickIterator = getBrickBuster().getBricks().iterator();
		while (!brickCollision && brickIterator.hasNext()) {
			Brick brick = brickIterator.next();
			Vector2[] vertices = brick.getVertices();
			for(int i = 0; i < vertices.length; i++) {
				Vector2 v1 = vertices[i];
				Vector2 v2 = vertices[i+1 < vertices.length? i+1 : 0];
				Vector2 segment = v2.cpy().sub(v1);
				Vector2 nearest = Utils.nearestPoint(v1.cpy(), segment, new_pos.cpy());

				if (nearest.dst(new_pos.x, new_pos.y) <= RADIUS) {
					brickIterator.remove();
					brick.hit();
					if (getBrickBuster().getBricks().size() <= BLOCKS_FOR_BOOST) {
						speed = BOOST_SPEED;
					}
					Utils.reflect(direction, segment.nor());
					brickCollision = true;
					break;
				}
			}
		}

		if (new_pos.x + RADIUS > BrickBuster.WIDTH || new_pos.x - RADIUS < 0) {
			Utils.reflect(direction, BrickBuster.VERTICAL_EDGE);
		} else if (new_pos.y + RADIUS > BrickBuster.HEIGHT) {
			Utils.reflect(direction, BrickBuster.HORIZONTAL_EDGE);
		} else if (new_pos.y + RADIUS < 0) {
			isDead = true;
			return;
		}

		if (direction.y < 0 && new_pos.y <= Paddle.PADDLE_Y + Paddle.PADDLE_HEIGHT + RADIUS) {
			Pair<Vector2, Vector2> paddle = getBrickBuster().getPaddle().getTopEdge();
			Vector2 lineDir = paddle.getValue().sub(paddle.getKey());
			Vector2 nearest = Utils.nearestPoint(paddle.getKey().cpy(), lineDir, new_pos.cpy());

			if (nearest.dst(new_pos.x, new_pos.y) <= RADIUS) {
				paddleCollision();
				if (getBrickBuster().getPaddle().isSticky()) {
					return;
				}
			}
		}

		getPos().add(direction.cpy().scl(speed * dt));
	}

	public Vector2 paddleReflectAngle() {
		Paddle paddle = getBrickBuster().getPaddle();
		float rel = MathUtils.clamp((getX() - paddle.getX()) + (paddle.getWidth()/2), 5, paddle.getWidth()-5);
		float newAngle = MathUtils.PI - (MathUtils.PI * (rel / paddle.getWidth()));
		return new Vector2(MathUtils.cos(newAngle), MathUtils.sin(newAngle));
	}

	public void launch() {
		if (getBrickBuster().getPaddle().isSticky()) {
			direction = paddleReflectAngle();
		} else {
			// launch at random angle between 135 and 45 degrees
			float angle = MathUtils.random(MathUtils.PI/2) + MathUtils.PI/4;
			direction = new Vector2(MathUtils.cos(angle), MathUtils.sin(angle));
		}
		isStuck = false;
	}

	public void paddleCollision() {
		Paddle paddle = getBrickBuster().getPaddle();
		if (paddle.isSticky()) {
			isStuck = true;
			setY(Paddle.PADDLE_Y + Paddle.PADDLE_HEIGHT + RADIUS);
			return;
		}
		direction = paddleReflectAngle();
	}


	public boolean isStuck() {
		return isStuck;
	}

	public boolean isDead() {
		return isDead;
	}
}
