package com.me.brickbuster.entity;

import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.glutils.ShapeRenderer.ShapeType;
import com.badlogic.gdx.math.Vector2;
import com.me.brickbuster.BrickBuster;
import com.me.brickbuster.entity.powerup.PowerUp;

public class Brick extends Entity {
	public static final Color BLOCK_COLOR = Color.FOREST;
	public static final int BLOCK_WIDTH = 50;
	public static final int BLOCK_HEIGHT = 20;

	private Class<? extends PowerUp> powerUpType;

	private Vector2[] vertices;

	public Brick(BrickBuster brickBuster, Class<? extends PowerUp> powerUpType, int x, int y) {
		super(brickBuster, x, y);
		this.powerUpType = powerUpType;

		this.vertices = new Vector2[] {
			new Vector2(x, y),
			new Vector2(x + BLOCK_WIDTH, y),
			new Vector2(x + BLOCK_WIDTH, y + BLOCK_HEIGHT),
			new Vector2(x, y + BLOCK_HEIGHT)
		};
	}

	@Override
	public void render() {
		getShapeRenderer().begin(ShapeType.Filled);
		getShapeRenderer().setColor(BLOCK_COLOR);
		getShapeRenderer().rect(getX(), getY(), BLOCK_WIDTH, BLOCK_HEIGHT);
		getShapeRenderer().end();
	}

	@Override
	public void update(float dt) {
	}

	public boolean hit() {
		if (powerUpType != null) {
			try {
				PowerUp powerUp = powerUpType.getConstructor(BrickBuster.class, Brick.class).newInstance(getBrickBuster(), this);
				getBrickBuster().addPowerUp(powerUp);
			} catch(Exception e) {

			}
		}
		return true;
	}

	public Vector2[] getVertices() {
		return vertices;
	}

}
